// Extend PrintWriter to SCREAM EVERYTHING that is printed using
// println() and print() methods.  Can toggle volume to scream/not
// scream.
import java.io.*;
public class ScreamWriter extends PrintWriter {
  private boolean highVolume;

  // Each constructor must call super(..) to build the parent class
  // state. This saves us the trouble of knowing how to set up output
  // for files and output streams
  public ScreamWriter(OutputStream o){
    super(o);
    this.highVolume = true;
  }
  public ScreamWriter(File f) throws Exception{
    super(f);
    this.highVolume = true;
  }
  public ScreamWriter(String filename) throws Exception{
    super(filename);
    this.highVolume = true;
  }

  // Override the printing methods to capitalize everything that is
  // printed to affect screaming.
  public void println(String s){
    String output = s;
    if(this.highVolume){
      output = output.toUpperCase();
    }
    super.println(output);
    this.flush();
  }

  public void toggleVolume(){
    this.highVolume = !this.highVolume;
  }
  
  // Override Equals
  public boolean equals(Object o){
    if(o instanceof ScreamWriter){
      ScreamWriter that = (ScreamWriter) o;
      return this.highVolume == that.highVolume;
    }
    else{
      return false;
    }
  }

}
